package com.fjhx.base.system;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.fjhx.tenant.entity.tenant.po.TenantInfo;
import com.fjhx.tenant.service.tenant.TenantInfoService;
import com.ruoyi.common.annotation.TenantIgnore;
import com.ruoyi.common.constant.BaseSourceConstant;
import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.core.domain.entity.SysMenu;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.domain.model.LoginBody;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.framework.web.service.SysLoginService;
import com.ruoyi.framework.web.service.SysPermissionService;
import com.ruoyi.system.service.ISysDeptService;
import com.ruoyi.system.service.ISysMenuService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.Set;

/**
 * 登录验证
 *
 * @author ruoyi
 */
@Slf4j
@DS(BaseSourceConstant.BASE)
@RestController
public class SysLoginController {

    @Autowired
    private SysLoginService loginService;

    @Autowired
    private ISysMenuService menuService;

    @Autowired
    private SysPermissionService permissionService;

    @Autowired
    private HttpServletRequest request;

    @Autowired
    private TenantInfoService tenantInfoService;
    @Autowired
    private ISysDeptService deptService;

    /**
     * 登录方法
     *
     * @param loginBody 登录信息
     * @return 结果
     */
    @TenantIgnore
    @PostMapping("/login")
    public AjaxResult login(@RequestBody LoginBody loginBody) {
        String tenantId = request.getHeader("tenantId");
        log.error("调用方法：login，tenantId：{}，date：{}，data；{}",
                tenantId, DateUtil.formatDateTime(new Date()), JSONObject.toJSONString(loginBody));

        TenantInfo tenantInfo = tenantInfoService.getOne(q -> q.eq(TenantInfo::getTenantId, tenantId));
        if (tenantInfo == null) {
            return AjaxResult.error("租户id不存在");
        }

        if (Objects.equals(tenantInfo.getStatus(), 0)) {
            return AjaxResult.error("租户已被禁用");
        }

        // 生成令牌
        String token = loginService.login(tenantId, loginBody.getUsername(), loginBody.getPassword(),
                loginBody.getCode(), loginBody.getUuid());

        AjaxResult ajax = AjaxResult.success();
        ajax.put(Constants.TOKEN, token);
        return ajax;
    }

    /**
     * 获取用户信息
     *
     * @return 用户信息
     */
    @GetMapping("getInfo")
    public AjaxResult getInfo() {
        SysUser user = SecurityUtils.getLoginUser().getUser();
        user.setPassword(null);

        //赋值关联归属公司列表
        List<SysDept> list = deptService.list(IWrapper.<SysDept>getWrapper().in(SysDept::getDeptId, SecurityUtils.getUserCompanyIds()));
        user.setCompanyList(list);
        //赋值归属公司信息
        SysDept company = deptService.getById(user.getCompanyId());
        if(ObjectUtil.isNotEmpty(company)) {
            user.setCompanyName(company.getDeptName());
        }

        // 角色集合
        Set<String> roles = permissionService.getRolePermission(user);
        // 权限集合
        Set<String> permissions = permissionService.getMenuPermission(user);
        AjaxResult ajax = AjaxResult.success();
        ajax.put("user", user);
        ajax.put("roles", roles);
        ajax.put("permissions", permissions);
        return ajax;
    }

    /**
     * 获取路由信息
     *
     * @return 路由信息
     */
    @GetMapping("getRouters")
    public AjaxResult getRouters() {
        Long userId = SecurityUtils.getUserId();
        List<SysMenu> menus = menuService.selectMenuTreeByUserId(userId);
        return AjaxResult.success(menuService.buildMenus(menus));
    }
}
