package com.fjhx.tenant.utils;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.extra.spring.SpringUtil;
import com.baomidou.dynamic.datasource.toolkit.DynamicDataSourceContextHolder;
import com.fjhx.tenant.entity.dict.dto.DictTenantDataSelectDto;
import com.fjhx.tenant.entity.dict.vo.DictTenantDataVo;
import com.fjhx.tenant.service.dict.DictTenantDataService;
import com.ruoyi.common.constant.BaseSourceConstant;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

public class DictUtils {

    private static final DictTenantDataService dictTenantDataService = SpringUtil.getBean(DictTenantDataService.class);

    /**
     * 获取字典map
     */
    public static Map<String, String> getDictMap(String code) {
        List<DictTenantDataVo> dictTenantDataServiceList = getDictList(code);
        Map<String, String> dictMap = new HashMap<>();
        if (ObjectUtil.isNotEmpty(dictTenantDataServiceList)) {
            dictMap = dictTenantDataServiceList.stream().collect(Collectors.toMap(DictTenantDataVo::getDictKey, DictTenantDataVo::getDictValue));
        }
        return dictMap;
    }

    public static Map<String, DictTenantDataVo> getDictDataMap(String code) {
        List<DictTenantDataVo> dictTenantDataServiceList = getDictList(code);
        Map<String, DictTenantDataVo> dictMap = dictTenantDataServiceList.stream()
                .collect(Collectors.toMap(DictTenantDataVo::getDictKey, Function.identity()));
        return ObjectUtil.defaultIfNull(dictMap, new HashMap<>());
    }

    /**
     * 获取字典List
     */
    public static List<DictTenantDataVo> getDictList(String code) {
        DictTenantDataSelectDto dto = new DictTenantDataSelectDto();
        dto.setDictCode(code);
        DynamicDataSourceContextHolder.push(BaseSourceConstant.BASE);
        List<DictTenantDataVo> dictTenantDataServiceList = dictTenantDataService.getList(dto);
        DynamicDataSourceContextHolder.poll();
        return dictTenantDataServiceList;
    }

    /**
     * 获取全部字典map
     */
    public static Map<String, Map<String, String>> getAllDictKVMap() {
        return dictTenantDataService.getAllDictKVMap();
    }
}
